#! /usr/bin/env python

import pygame, os, random
from pygame.locals import *
from pygame.color import Color

from albow.screen import Screen
from albow.grid_view import GridView
from albow.controls import Label, Button, Image, ImageButton
from albow.layout import Column

from albow.image_array import get_image_array
from albow.resource import get_font, get_image
from albow.dialogs import alert

class TableGridView(GridView):
    column_number = 24
    row_number = 12
    def __init__(self, game_status):
        GridView.__init__(self, (18, 24), self.row_number, self.column_number)
        self.game_status = game_status

    def num_rows(self):
        return self.row_number
    
    def num_cols(self):
        return self.column_number
    
    def draw_cell(self, surface, row, col, rect):
        task_no = self.game_status.get_table_status((row, col))
        if task_no > 0:
            col = self.game_status.get_task()[task_no-1][1]
            surface.fill(col, rect)
        if task_no == -1:
            surface.fill((64,64,64),rect)
        
    def click_cell(self, row, col, event):
        if self.game_status.get_member()[row] != False:
            if self.game_status.current_turn-1 <= col:
                self.game_status.set_table_status([row,col],self.game_status.current_task+1)
        
class PalletGridView(GridView):
    column_number = 1
    row_number = 12
    def __init__(self, max):
        GridView.__init__(self, (64, 24), self.row_number, self.column_number)
        self.result = 0
        self.max = max
    def num_rows(self):
        return self.row_number
    
    def num_cols(self):
        return self.column_number
    
    def draw_cell(self, surface, row, col, rect):
        False
        
    def click_cell(self, row, col, event):
        if row < self.max:
            self.result = row
    def get_result(self):
        return self.result
        
class MemberGridView(GridView):
    column_number = 1
    row_number = 12
    def __init__(self):
        GridView.__init__(self, (72, 24), self.row_number, self.column_number)
    
    def num_rows(self):
        return self.row_number
    
    def num_cols(self):
        return self.column_number
    
    def draw_cell(self, surface, row, col, rect):
        False
        
    def click_cell(self, row, col, event):
        False

class WorkTable(Screen):
    def __init__(self, shell, resource, game_status):
        self.blink_count = 255
        self.blink_speed = -8

        Screen.__init__(self, shell)
        self.shell = shell
        self.resource = resource
        self.game_status = game_status

        self.grid = TableGridView(self.game_status)
        self.pallet_grid = PalletGridView(len(self.game_status.get_task()))
        self.member_grid = MemberGridView()
        
    def enter_screen(self):
        self.grid.rect.topleft = (106, 96)
        self.add(self.grid)

        self.pallet_grid.rect.topleft = (534+18, 80+16)
        self.add(self.pallet_grid)

        self.member_grid.rect.topleft = (106-72, 96)
        self.add(self.member_grid)
        
        next_button = ImageButton("next_turn_button.png", action = self.turn_end)
        next_button.rect.topleft = (534-32, 480-48)
        self.add(next_button)

    def turn_end(self):
        self.resource.next_sound.play()
        for i in xrange(0,12):
            if self.game_status.get_table_status((i,self.game_status.current_turn-1)) > 0:
                self.game_status.set_table_status((i,self.game_status.current_turn-1), -1)
        self.game_status.current_turn += 1
        if self.game_status.current_turn > 24:
            self.game_end()

    def game_end(self):
        alert("Thank you for playing!")
        self.shell.quit()

    def draw(self, surface):
        surface.blit(self.resource.bg_theme, (-640, 0))
 
        # Cash Area
        cash_image = get_image("dollar.png")
        surface.blit(cash_image, (640-88-16, 0))
               
        member = self.game_status.get_member()
        font_d = get_font(14, "Dekar.otf")
        n = 0
        for i in member:
            if i != False:
                text = font_d.render(i[0], 1, (255, 255, 255))
                surface.blit(text, (106-72+6, 96+6+24*n))
            n += 1

        task = self.game_status.get_task()
        font_d = get_font(12, "Dekar.otf")
        n = 0
        for i in task:
            text = font_d.render(i[0], 1, (255,255,255))
            surface.fill(i[1], (534+18, 80+16+24*n, 64, 24))
            surface.blit(text, (534+18+6, 80+16+6+24*n))
            n += 1
        self.game_status.current_task = self.pallet_grid.get_result()
        pygame.draw.rect(surface, (self.blink_count, self.blink_count, self.blink_count), (534+18-4, 80+16+24*self.game_status.current_task, 64+8, 24), 2) 

        # Cursor
        ct  = self.game_status.current_turn-1
        points = [[106+ct*18, 96-32-self.blink_count/10], [106+ct*18+16, 96-32-self.blink_count/10], [106+ct*18+8, 96-16-self.blink_count/10]]
        pygame.draw.polygon(surface, (192, 128, 128), points)
        text = font_d.render("Current", 1, (255,255,255))
        surface.blit(text, (106+ct*18-8, 96-32-12-self.blink_count/10))

    def begin_frame(self):
        self.blink_count += self.blink_speed
        if self.blink_count > 255:
            self.blink_count = 255
            self.blink_speed = -self.blink_speed
        elif self.blink_count <192:
            self.blink_count = 192
            self.blink_speed = -self.blink_speed
        self.invalidate()

    def quit(self):
        sys.exit(0)
